@@ -22,31 +22,31 @@ testthat::test_file("tests/manual/test-code/test-checkPopDynamics.R") # ok
 
 testthat::test_file("tests/manual/test-code/test-Data2csv.R") # ok
 
-testthat::test_file("tests/manual/test-code/test-Data_Functions.R") #  ok 
+testthat::test_file("tests/manual/test-code/test-Data_Functions.R") # ok
 
 testthat::test_file("tests/manual/test-code/test-Data_Plotting.R") # ok
 
-testthat::test_file("tests/manual/test-code/test-Fease_Functions.R") # ok 
+testthat::test_file("tests/manual/test-code/test-Fease_Functions.R") #  
 
 testthat::test_file("tests/manual/test-code/test-Import_Data.R") # ok
 
 testthat::test_file("tests/manual/test-code/test-MPs.R") # ok
 
-testthat::test_file("tests/manual/test-code/test-MSE_functions.R") # ok
+testthat::test_file("tests/manual/test-code/test-MSE_functions.R") # ok 
 
 testthat::test_file("tests/manual/test-code/test-MSE_Plotting.R") # cosewic and VOI errors
 
-testthat::test_file("tests/manual/test-code/test-OM_functions.R") # ok
+testthat::test_file("tests/manual/test-code/test-OM_functions.R") # ok 
 
-testthat::test_file("tests/manual/test-code/test-OM_init_doc.R") # ok 
+testthat::test_file("tests/manual/test-code/test-OM_init_doc.R") # ok
 
-testthat::test_file("tests/manual/test-code/test-OM_Plotting.R") # ok (ish)
+testthat::test_file("tests/manual/test-code/test-OM_Plotting.R") # takes ages!  ok
 
-testthat::test_file("tests/manual/test-code/test-PMobjects.R") # ok
+testthat::test_file("tests/manual/test-code/test-PMobjects.R") # ok  
 
 testthat::test_file("tests/manual/test-code/test-RealIndices.R") # ok
 
-testthat::test_file("tests/manual/test-code/test-runMSE.R") # 
+testthat::test_file("tests/manual/test-code/test-runMSE.R") # ok
 
-testthat::test_file("tests/manual/test-code/test-slotDescription.R") # 
+testthat::test_file("tests/manual/test-code/test-slotDescription.R") # ok
 
